'use strict';

var stringToArray = function stringToArray(string) {
  return string.match(/[\uD800-\uDBFF][\uDC00-\uDFFF]|[^\uD800-\uDFFF]/g) || [];
};

var inverseObject = function inverseObject(obj) {
  var inversed = Object.keys(obj).map(function (key) {
    return [key, obj[key]];
  }).reduce(function (m, pair) {
    m[pair[1]] = pair[0];
    return m;
  }, {});

  if (Object.keys(obj).length !== Object.keys(inversed).length) {
    throw new Error('The same key exists and lost information.');
  }

  return inversed;
};

var ONE_LETTER_OF_ALPHABET = {
  a: 'ア',
  b: 'カ',
  c: 'キ',
  d: 'ク',
  e: 'イ',
  f: 'ケ',
  g: 'コ',
  h: 'サ',
  i: 'ウ',
  j: 'シ',
  k: 'ス',
  l: 'セ',
  m: 'ソ',
  n: 'タ',
  o: 'エ',
  p: 'チ',
  q: 'ツ',
  r: 'テ',
  s: 'ト',
  t: 'ナ',
  u: 'オ',
  v: 'ニ',
  w: 'ヌ',
  x: 'ネ',
  y: 'ノ',
  z: 'ハ'
};
var ONE_LETTER_OF_ALPHABET_E_TO_T = ONE_LETTER_OF_ALPHABET;
var ONE_LETTER_OF_ALPHABET_T_TO_E = inverseObject(ONE_LETTER_OF_ALPHABET);

var TWO_LETTERS_OF_ALPHABET = {
  ce: 'ゲ',
  de: 'レ',
  ea: 'ユ',
  er: 'ヨ',
  gr: 'ザ',
  la: 'ヤ',
  lo: 'モ',
  me: 'ヘ',
  mo: 'ビ',
  na: 'ラ',
  ne: 'リ',
  pa: 'ル',
  pl: 'フ',
  re: 'メ',
  ri: 'ミ',
  yo: 'ホ',
  wh: 'ム'
};
var TWO_LETTERS_OF_ALPHABET_E_TO_T = TWO_LETTERS_OF_ALPHABET;
var TWO_LETTERS_OF_ALPHABET_T_TO_E = inverseObject(TWO_LETTERS_OF_ALPHABET);

var TOKEN_TYPES = [{
  type: 'properName',
  pattern: /^('[^']+?'|"[^"]+?")/
}, {
  type: 'word',
  pattern: /^[^\s]+/
}, {
  type: 'spaces',
  pattern: /^\s+/
}];

var lexer = function lexer(text) {
  var tokens = [];
  var restText = text;

  while (restText !== '') {
    var analyzeSuccessed = false;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = TOKEN_TYPES[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        var _ref = _step.value;
        var pattern = _ref.pattern,
            type = _ref.type;

        var matched = pattern.exec(restText);
        if (matched !== null) {
          restText = restText.replace(pattern, '');
          analyzeSuccessed = true;
          tokens.push({
            type: type,
            text: matched[0]
          });
          break;
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    if (analyzeSuccessed === false) {
      throw new Error('Lexical analysis failed. This is an unexpected error.');
    }
  }

  return tokens;
};

module.exports = {
  ONE_LETTER_OF_ALPHABET_E_TO_T: ONE_LETTER_OF_ALPHABET_E_TO_T,
  ONE_LETTER_OF_ALPHABET_T_TO_E: ONE_LETTER_OF_ALPHABET_T_TO_E,
  TOKEN_TYPES: TOKEN_TYPES,
  TWO_LETTERS_OF_ALPHABET_E_TO_T: TWO_LETTERS_OF_ALPHABET_E_TO_T,
  TWO_LETTERS_OF_ALPHABET_T_TO_E: TWO_LETTERS_OF_ALPHABET_T_TO_E,
  lexer: lexer,
  stringToArray: stringToArray
};